Type: Package
Package: SpaDES.tools
Title: Additional Tools for Developing Spatially Explicit Discrete
        Event Simulation (SpaDES) Models
Version: 2.1.1
Date: 2026-01-09
Authors@R: c(
    person("Eliot J B", "McIntire", , "eliot.mcintire@nrcan-rncan.gc.ca", role = "aut",
           comment = c(ORCID = "0000-0002-6914-8316")),
    person("Alex M", "Chubaty", , "achubaty@for-cast.ca", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7146-8135")),
    person("Yong", "Luo", , "Yong.Luo@gov.bc.ca", role = "ctb"),
    person("Ceres", "Barros", , "ceres.barros@ubc.ca", role = "ctb",
           comment = c(ORCID = "0000-0003-4036-977X")),
    person("Steve", "Cumming", , "Steve.Cumming@sbf.ulaval.ca", role = "ctb"),
    person("Jean", "Marchal", , "jean.d.marchal@gmail.com", role = "ctb"),
    person("His Majesty the King in Right of Canada, as represented by the Minister of Natural Resources Canada", role = "cph")
  )
Description: Provides GIS and map utilities, plus additional modeling
    tools for developing cellular automata, dynamic raster models, and
    agent based models in 'SpaDES'.  Included are various methods for
    spatial spreading, spatial agents, GIS operations, random map
    generation, and others.  See '?SpaDES.tools' for an categorized
    overview of these additional tools.  The suggested package 'NLMR' can
    be installed from the following repository:
    (<https://PredictiveEcology.r-universe.dev>).
License: GPL-3
URL: https://spades-tools.predictiveecology.org,
        https://github.com/PredictiveEcology/SpaDES.tools
BugReports: https://github.com/PredictiveEcology/SpaDES.tools/issues
Depends: R (>= 4.3)
Imports: backports, checkmate (>= 1.8.2), data.table (>= 1.10.4),
        fpCompare (>= 0.2.1), graphics, methods, parallel, Rcpp (>=
        0.12.12), reproducible (>= 2.1.3), stats, terra
Suggests: animation, bit (>= 1.1-12), covr, DEoptim (>= 2.2-4), dqrng,
        fastmatch, knitr, NLMR (>= 1.1.1), quickPlot (>= 1.0.2), raster
        (>= 2.5-8), rmarkdown, sf, snow, sp (>= 1.2-4), testthat (>=
        3.0.0), tools, withr
LinkingTo: Rcpp
Additional_repositories: https://predictiveecology.r-universe.dev/
ByteCompile: yes
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-CA
RoxygenNote: 7.3.3
Collate: 'RcppExports.R' 'heading.R' 'SELES.R'
        'distanceFromEachPoint.R' 'environment.R' 'helpers.R'
        'initialize.R' 'mapReduce.R' 'mergeRaster.R' 'movement.R'
        'neighbourhood.R' 'numerical-comparisons.R' 'probability.R'
        'resample.R' 'rings.R' 'spades-tools-deprecated.R'
        'spades-tools-package.R' 'splitRaster.R' 'spread.R' 'spread2.R'
        'spread3.R' 'studyArea.R' 'zzz.R'
NeedsCompilation: yes
Packaged: 2026-01-09 18:37:29 UTC; achubaty
Author: Eliot J B McIntire [aut] (ORCID:
    <https://orcid.org/0000-0002-6914-8316>),
  Alex M Chubaty [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7146-8135>),
  Yong Luo [ctb],
  Ceres Barros [ctb] (ORCID: <https://orcid.org/0000-0003-4036-977X>),
  Steve Cumming [ctb],
  Jean Marchal [ctb],
  His Majesty the King in Right of Canada, as represented by the Minister
    of Natural Resources Canada [cph]
Maintainer: Alex M Chubaty <achubaty@for-cast.ca>
Repository: CRAN
Date/Publication: 2026-01-11 06:11:56 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 22:30:33 UTC; unix
Archs: SpaDES.tools.so.dSYM
