# Generated by roxygen2: do not edit by hand

S3method(randomPolygon,default)
S3method(testEquivalentMetadata,default)
export(.pointDistance)
export(adj)
export(agentLocation)
export(cir)
export(crw)
export(directionFromEachPoint)
export(distanceFromEachPoint)
export(dwrpnorm2)
export(fastCrop)
export(gaussMap)
export(heading)
export(inRange)
export(initiateAgents)
export(mergeRaster)
export(middlePixel)
export(move)
export(neutralLandscapeMap)
export(numAgents)
export(probInit)
export(randomPolygon)
export(randomPolygons)
export(randomStudyArea)
export(rasterizeReduced)
export(rings)
export(runifC)
export(specificNumPerPatch)
export(splitRaster)
export(spokes)
export(spread)
export(spread2)
export(spread3)
export(testEquivalentMetadata)
export(transitions)
export(wrap)
exportMethods(mergeRaster)
import(methods)
import(utils)
importFrom(Rcpp,evalCpp)
importFrom(checkmate,assert)
importFrom(checkmate,assertClass)
importFrom(checkmate,assertMultiClass)
importFrom(checkmate,assertNumeric)
importFrom(checkmate,checkClass)
importFrom(checkmate,checkDataTable)
importFrom(checkmate,checkLogical)
importFrom(checkmate,checkMultiClass)
importFrom(checkmate,checkNumeric)
importFrom(checkmate,checkScalarNA)
importFrom(checkmate,qassert)
importFrom(data.table,":=")
importFrom(data.table,alloc.col)
importFrom(data.table,as.data.table)
importFrom(data.table,copy)
importFrom(data.table,data.table)
importFrom(data.table,is.data.table)
importFrom(data.table,key)
importFrom(data.table,rbindlist)
importFrom(data.table,set)
importFrom(data.table,setattr)
importFrom(data.table,setcolorder)
importFrom(data.table,setkey)
importFrom(data.table,setkeyv)
importFrom(data.table,setnames)
importFrom(data.table,uniqueN)
importFrom(fpCompare,"%<<%")
importFrom(fpCompare,"%<=%")
importFrom(fpCompare,"%==%")
importFrom(fpCompare,"%>=%")
importFrom(fpCompare,"%>>%")
importFrom(graphics,par)
importFrom(parallel,clusterApply)
importFrom(parallel,clusterApplyLB)
importFrom(parallel,mclapply)
importFrom(reproducible,.isGridded)
importFrom(reproducible,checkPath)
importFrom(reproducible,maxFn)
importFrom(reproducible,minFn)
importFrom(stats,dweibull)
importFrom(stats,na.omit)
importFrom(stats,pexp)
importFrom(stats,pweibull)
importFrom(stats,rbeta)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(terra,"crs<-")
importFrom(terra,"values<-")
importFrom(terra,align)
importFrom(terra,cellFromXY)
importFrom(terra,compareGeom)
importFrom(terra,crds)
importFrom(terra,crop)
importFrom(terra,crs)
importFrom(terra,distance)
importFrom(terra,ext)
importFrom(terra,extend)
importFrom(terra,extract)
importFrom(terra,focal)
importFrom(terra,freq)
importFrom(terra,geomtype)
importFrom(terra,is.related)
importFrom(terra,levels)
importFrom(terra,merge)
importFrom(terra,mosaic)
importFrom(terra,ncell)
importFrom(terra,ncol)
importFrom(terra,nrow)
importFrom(terra,origin)
importFrom(terra,project)
importFrom(terra,rast)
importFrom(terra,res)
importFrom(terra,setValues)
importFrom(terra,spatSample)
importFrom(terra,unwrap)
importFrom(terra,values)
importFrom(terra,vect)
importFrom(terra,wrap)
importFrom(terra,writeRaster)
importFrom(terra,xmax)
importFrom(terra,xmin)
importFrom(terra,xres)
importFrom(terra,xyFromCell)
importFrom(terra,ymax)
importFrom(terra,ymin)
importFrom(terra,yres)
importFrom(utils,assignInMyNamespace)
useDynLib(SpaDES.tools, .registration = TRUE)
